#!/usr/bin/env python

import os, gobject, pango, pygtk, gtk, random, urllib
from manager import manager as manager

PLAYLIST = os.path.expanduser("~/.kagu/playlist")


#
#  Provides a playlist store to queue player jobs.
#
class Playlist:
  
  list, current, history = [], None, []
  continuous, random, repeat, repeat_one = True, False, False, False
  loadcount, loadcur = 0, 0
  
  #
  #  Creates the playlist widgets and adds them to vbox.
  #
  def __init__(self):
    self.continuous = manager.prefs.getBool("continuous")
    self.random = manager.prefs.getBool("random")
    self.repeat = manager.prefs.getBool("repeat")
    self.repeat_one = manager.prefs.getBool("repeat_one")
    
  #
  #  Update
  #
  def update(self):
    # update nowplaying indicator
    manager.nowplaying_view.update()
    manager.play_button.update()
    return True

  #
  #  Plays the specified target from the playlist.
  #
  def play(self, index, log=True, event=None):
    if log and not self.is_empty():  #append current target to history
      self.history.append(self.current)
    
    #if event:  #clear selection, select target, show info
   
    print "index=%s" % (index,)
    self.current = index
    if manager.scrobbler: manager.scrobbler.play()
    manager.player.play(self.status())  #begin playing target
    self.update()
    return True

  def pause(self):
    manager.player.pause()

  def is_paused(self):
    return manager.player.paused
    
  #
  #  Plays a target step rows from the currently active target.
  #
  def jump(self, step, log=True, event=None):
    print "jump: list='%s',current='%s',random='%s'" % (self.list,self.current,self.random)
    
    if not self.list:  #empty list
      return True
    
    if self.random:  #disregard step, use random
      i = random.randint(0, len(self.list) - 1)
    elif not self.is_empty():
      i = self.current + step
    else:
      i = step
   
    print "jump: step='%s'" % (step,)

    if i > len(self.list) - 1: i = 0
    if i < 0: i = len(self.list) + i
   
    print "step=%d,i=%d,len=%d,current=%d" % (step,i,len(self.list),self.current)
    return self.play(i, log, event)
    
  #
  #  Stops the current player job and prevents a jump.
  #
  def stop(self, widget, event):
    if manager.scrobbler: manager.scrobbler.stop()
    manager.player.close()
    return True
    
  #
  #  Returns True if the playlist has been exhausted, False otherwise.
  #
  def exhausted(self):
    if self.repeat:  #never exhausted
      return False
   
    if self.current >= len(self.list) - 1:
      return True
    return False
  
  #
  #  Plays a previously played target if available, jumps otherwise.
  #
  def prev(self, widget, event):
    index = None
    while not index and len(self.history):
      index = self.history.pop()
    
    if index:  #play target from history
      return self.play(index, False, event)
    
    return self.jump(-1, False, event)
    
  #
  #  Plays the next available target.
  #
  def next(self, widget, event):
    if not event and self.exhausted():  #exhausted list, return
      return self.stop(None, None)  #reset window title, etc
    elif self.repeat_one:
      return self.jump(0, True, event)
    return self.jump(1, True, event)
    
  #
  #  Returns the title of the current target, or None.
  #
  def status(self):
    if self.is_empty():
      return None
    return self.list[self.current]
  
  #
  #  Saves playlist to the specified file, defaulting to PLAYLIST.
  #
  def save(self, filename=PLAYLIST):
    try:
      f = open(filename, "w")
    except StandardError:
      return False
    
    for target in self.list:  #append each target to file
      f.write(target + "\n")
      
    f.close()
    return True
  
  #
  #  Loads the specified targets (lazily) to the playlist.
  #
  def load(self, targets):
    self.loadcount += len(targets)
    
    for t in targets:
      target = urllib.unquote(t).replace("file://", "")
      
    self.add(target)
    return True
  
  #
  #  Adds the specified target to the playlist.
  #
  def add(self, target):
    self.loadcur += 1
    
    if target.find("://") > -1:  #locations, http://, dvd://
      name = target
    else:  #normal files
      name = target

    isem = self.is_empty()
    
    self.list.append(name)
    manager.nowplaying_view.dirty = True

    if isem: self.play(0)

    return False  #cancel idle callback
    
  #
  #  Stops playback and removes all targets from playlist.
  #
  def clear(self):
    if self.current!=None:  #stop if necessary
      self.stop(None, None)
      self.current = None
      self.history = []

    self.list = []
    manager.nowplaying_view.dirty = True
    return True

  #
  #  Is the playlist empty?
  #
  def is_empty(self):
    return((not self.list) or (len(self.list)==0))
